<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                exclude-result-prefixes="xalan">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="*"/>

    <xsl:variable name="debug" select="false()" />

    <xsl:template match="/parameters">
        <xsl:if test="$debug">
            <xsl:message terminate="no">Debug from ChartOptions.xsl</xsl:message>
            <xsl:message terminate="no">    Input : <xsl:copy-of select="/" /></xsl:message>
        </xsl:if>
        <options>
            <xsl:apply-templates select="stacking|axis"/> <!-- To be deleted with ChartDeprecatedRenderer -->
            <xsl:if test="not(stacking)">
                <xsl:attribute name="stacking"><xsl:value-of select="'none'" /></xsl:attribute>
            </xsl:if>
            <xsl:choose>
                <xsl:when test="version=1">
                    <xsl:apply-templates select="type" /> <!-- To be deleted with ChartDeprecatedRenderer -->
                    <xsl:apply-templates select="drilldown" /> <!-- To be deleted with ChartDeprecatedRenderer -->
                </xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="options/options"/>
                    <xsl:if test="not(options/options)">
                        <xsl:call-template name="NOOPTIONS" />
                    </xsl:if>
                </xsl:otherwise>
            </xsl:choose>
        </options>
    </xsl:template>

    <xsl:template match="type"><!-- To be deleted with ChartDeprecatedRenderer -->
        <xsl:attribute name="type"><xsl:value-of select="text()"/></xsl:attribute>
    </xsl:template>
    <xsl:template match="stacking"><!-- To be deleted with ChartDeprecatedRenderer -->
        <xsl:attribute name="stacking">
            <xsl:choose>
                <xsl:when test="text()='none' or text()='normal' or text()='percent'"><xsl:value-of select="text()"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="'none'"/></xsl:otherwise>
            </xsl:choose>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="axis"><!-- To be deleted with ChartDeprecatedRenderer -->
        <xsl:attribute name="axis"><xsl:value-of select="text()"/></xsl:attribute>
    </xsl:template>

    <xsl:template match="xAxis"><!-- To be deleted with ChartDeprecatedRenderer -->
        <xsl:copy>
            <xsl:for-each select="entry">
                <column><xsl:value-of select="text()"/></column>
            </xsl:for-each>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="yAxis|yAxis2|yAxis3"><!-- To be deleted with ChartDeprecatedRenderer -->
        <xsl:copy>
            <xsl:attribute name="id">
                <xsl:apply-templates select="." mode="POSITION"/>
            </xsl:attribute>
            <xsl:for-each select="entry">
                <column>
                    <xsl:for-each select="/parameters/drilldown/entry[@key=current()/text()]">
                        <xsl:attribute name="drilldown"><xsl:value-of select="text()"/></xsl:attribute>
                    </xsl:for-each>
                    <xsl:value-of select="text()"/>
                </column>
            </xsl:for-each>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="types|drilldown"><!-- To be deleted with ChartDeprecatedRenderer -->
        <xsl:copy-of select="."/>
    </xsl:template>

    <xsl:template match="options/options">
        <xsl:if test="yAxis1 or /parameters/yAxis1">
            <xsl:message terminate="yes">Element not authorized "yAxis1" use "yAxis" instead.</xsl:message>
        </xsl:if>
        <sections>
            <xsl:apply-templates select="./*" mode="OPTIONS"/>
            <xsl:if test="not(chart)"> <xsl:apply-templates select="." mode="OPTIONSCHART" /> </xsl:if>
            <xsl:if test="not(series)">
                <xsl:if test="not(/parameters/yAxis/entry or /parameters/yAxis2/entry or /parameters/yAxis3/entry)">
                    <section id="series" mode="unit" />
                </xsl:if>
                <xsl:apply-templates select="/parameters/yAxis|/parameters/yAxis2|/parameters/yAxis3" mode="OPTIONSSERIES" />
            </xsl:if>
            <xsl:if test="not(xAxis)"> <xsl:apply-templates select="." mode="OPTIONSXAXIS" /> </xsl:if>
            <xsl:if test="not(yAxis or yAxis2 or yAxis3)">
                <xsl:if test="not(/parameters/yAxis/entry or /parameters/yAxis2/entry or /parameters/yAxis3/entry)">
                    <xsl:apply-templates select="/parameters/types" mode="OPTIONSYAXIS" />
                </xsl:if>
                <xsl:apply-templates select="/parameters/yAxis[entry]|/parameters/yAxis2[entry]|/parameters/yAxis3[entry]" mode="OPTIONSYAXIS" />
            </xsl:if>
            <xsl:if test="not(plotOptions) and /parameters/stacking and /parameters/types/entry[@key='*']">
                <xsl:apply-templates select="/parameters/stacking" mode="OPTIONSPLOTOPTIONS" />
            </xsl:if>
        </sections>
    </xsl:template>

    <xsl:template match="chart" mode="OPTIONS">
        <section id="chart">
            <xsl:if test="not(./type) and /parameters/types/*[@key='*']">
                <option name="type"><xsl:value-of select="/parameters/types/*[@key='*']/text()" /></option>
            </xsl:if>
            <xsl:if test="not(./inverted) and /parameters/axis">
                <xsl:choose>
                    <xsl:when test="/parameters/axis='inverted'">
                        <option name="inverted">true</option>
                    </xsl:when>
                    <xsl:when test="/parameters/axis='polar'">
                        <option name="polar">true</option>
                    </xsl:when>
                </xsl:choose>
            </xsl:if>
            <xsl:if test="not(./text()=./*)">
                <xsl:apply-templates select=" ./*" mode="OPTIONS" />
            </xsl:if>
        </section>
    </xsl:template>
    <xsl:template match="series" mode="OPTIONS">
        <section id="series">
            <xsl:if test="@name">
                <xsl:variable name="name" select="@name" />
                <xsl:attribute name="name"><xsl:value-of select="$name" /></xsl:attribute>
                <xsl:choose>
                    <xsl:when test="@mode">
                        <xsl:attribute name="mode"><xsl:value-of select="@mode" /></xsl:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:attribute name="mode">
                            <xsl:choose>
                                <xsl:when test="/parameters/yAxis/entry[text()=$name]|/parameters/yAxis2/entry[text()=$name]|/parameters/yAxis3/entry[text()=$name]"><xsl:value-of select="'column'"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="'unit'" /></xsl:otherwise>
                            </xsl:choose>
                        </xsl:attribute>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:if test="not(type) and /parameters/types/entry[@key=$name]">
                    <option name="type"><xsl:value-of select="/parameters/types/entry[@key=$name]/text()" /></option>
                </xsl:if>
                <xsl:if test="not(yAxis)">
                    <xsl:choose>
                        <xsl:when test="/parameters/yAxis/entry or /parameters/yAxis2/entry or /parameters/yAxis3/entry">
                            <option name="yAxis">
                                <xsl:apply-templates select="/parameters/yAxis[entry[text()=$name]]|/parameters/yAxis2[entry[text()=$name]]|/parameters/yAxis3[entry[text()=$name]]" mode="SERIESYPOSITION" />
                            </option>
                        </xsl:when>
                        <xsl:when test="not(/parameters/yAxis/entry or /parameters/yAxis2/entry or /parameters/yAxis3/entry) and /parameters/types[entry[@key=$name]]">
                            <option name="yAxis">0</option>
                        </xsl:when>
                        <xsl:otherwise><option name="yAxis">0</option></xsl:otherwise>
                    </xsl:choose>
                </xsl:if>
            </xsl:if>
            <xsl:apply-templates select="./*[not(local-name()='type')]" mode="OPTIONS" />
        </section>
    </xsl:template>
    <xsl:template match="xAxis" mode="OPTIONS">
        <section id="xAxis">
            <xsl:if test="not(./category)">
                <xsl:if test="/parameters/xAxis/entry">
                    <option name="categories" mode="column"><xsl:value-of select="/parameters/xAxis/entry/text()" /></option>
                </xsl:if>
            </xsl:if>
            <xsl:apply-templates select="./*" mode="OPTIONS" />
        </section>
    </xsl:template>
    <xsl:template match="yAxis|yAxis2|yAxis3" mode="OPTIONS">
        <xsl:variable name="position"><xsl:apply-templates select="." mode="POSITION"/></xsl:variable>
        <section id="yAxis">
            <xsl:attribute name="position"><xsl:value-of select="$position" /></xsl:attribute>
            <xsl:apply-templates select="./*" mode="OPTIONS"/>
            <xsl:if test="not(opposite) and $position>1">
                <option name="opposite">true</option>
            </xsl:if>
        </section>
    </xsl:template>
    <xsl:template match="plotOptions" mode="OPTIONS">
        <section id="plotOptions" type="{@type}">
            <xsl:if test="@type=/parameters/type/entry[@key='*']/text() and (not(./stacking) and /parameters/stacking)">
                <option name="stacking">
                    <xsl:choose>
                        <xsl:when test="/parameters/stacking='normal' or /parameters/stacking ='percent'"><xsl:value-of select="/parameters/stacking" /></xsl:when>
                        <xsl:otherwise><xsl:value-of select="'null'" /></xsl:otherwise>
                    </xsl:choose>
                </option>
            </xsl:if>
            <xsl:apply-templates select="./*" mode="OPTIONS" />
        </section>
    </xsl:template>
    <xsl:template match="options/title" mode="OPTIONS">
        <section id="title">
            <xsl:apply-templates select="./*" mode="OPTIONS" />
            <xsl:if test="not(text)"> <option name="text">Untitled</option> </xsl:if>
        </section>
    </xsl:template>

    <xsl:template match="category" mode="OPTIONS">
        <option name="categories">
            <xsl:if test="@mode">
                <xsl:attribute name="mode"><xsl:value-of select="@mode" /></xsl:attribute>
            </xsl:if>
            <xsl:value-of select="text()"/>
        </option>
    </xsl:template>

    <xsl:template name="NOOPTIONS">
        <sections>
            <xsl:apply-templates select="." mode="OPTIONSCHART" />
            <xsl:choose>
                <xsl:when test="/parameters/yAxis[entry]|/parameters/yAxis2[entry]|/parameters/yAxis3[entry]">
                    <xsl:apply-templates select="/parameters/yAxis|/parameters/yAxis2|/parameters/yAxis3" mode="OPTIONSSERIES" />
                </xsl:when>
                <xsl:otherwise><section id="series" mode="unit" /></xsl:otherwise>
            </xsl:choose>
            <xsl:apply-templates select="." mode="OPTIONSXAXIS" />
            <xsl:choose>
                <xsl:when test="/parameters/yAxis[entry]|/parameters/yAxis2[entry]|/parameters/yAxis3[entry]">
                    <xsl:apply-templates select="/parameters/yAxis[entry]|/parameters/yAxis2[entry]|/parameters/yAxis3[entry]" mode="OPTIONSYAXIS" />
                </xsl:when>
                <xsl:when test="not(/parameters/yAxis/entry or /parameters/yAxis2/entry or /parameters/yAxis3/entry)">
                    <xsl:apply-templates select="/parameters/types" mode="OPTIONSYAXIS" />
                </xsl:when>
            </xsl:choose>
            <xsl:if test="/parameters/stacking and /parameters/types/entry[@key='*']">
                <xsl:apply-templates select="/parameters/stacking" mode="OPTIONSPLOTOPTIONS" />
            </xsl:if>
        </sections>
    </xsl:template>

    <xsl:template match="yAxis|yAxis2|yAxis3" mode="OPTIONSSERIES">
        <xsl:for-each select="entry">
            <xsl:variable name="series" select="text()" />
            <section id="series">
                <xsl:attribute name="name"><xsl:value-of select="$series" /></xsl:attribute>
                <xsl:attribute name="mode"><xsl:value-of select="'column'" /></xsl:attribute>
                <xsl:if test="/parameters/types/entry[@key=$series]"> <option name="type"><xsl:value-of select="/parameters/types/entry[@key=$series]/text()" /></option> </xsl:if>
                <option name="yAxis">
                    <xsl:apply-templates select="ancestor-or-self::yAxis|ancestor-or-self::yAxis2|ancestor-or-self::yAxis3" mode="SERIESYPOSITION" />
                </option>
            </section>
        </xsl:for-each>
    </xsl:template>
    <xsl:template match="yAxis|yAxis2|yAxis3" mode="OPTIONSYAXIS">
        <xsl:variable name="position"><xsl:apply-templates select="." mode="POSITION" /></xsl:variable>
        <section id="yAxis">
            <xsl:attribute name="position"><xsl:value-of select="$position" /></xsl:attribute>
            <xsl:if test="$position>1"><option name="opposite">true</option></xsl:if>
        </section>
    </xsl:template>
    <xsl:template match="stacking" mode="OPTIONSPLOTOPTIONS">
        <section id="plotOptions">
            <xsl:attribute name="type"><xsl:value-of select="/parameters/types/entry[@key='*']/text()" /></xsl:attribute>
            <option name="stacking">
                <xsl:choose>
                    <xsl:when test=".='normal' or .='percent'"><xsl:value-of select="." /></xsl:when>
                    <xsl:otherwise><xsl:value-of select="'null'" /></xsl:otherwise>
                </xsl:choose>
            </option>
        </section>
    </xsl:template>

    <xsl:template match="*" mode="OPTIONSCHART" priority="-1">
        <xsl:if test="/parameters/types/*[@key='*'] or /parameters/axis">
            <section id="chart">
                <xsl:if test="/parameters/types/*[@key='*']">
                    <option name="type"><xsl:value-of select="/parameters/types/*[@key='*']/text()" /></option>
                </xsl:if>
                <xsl:if test="/parameters/axis">
                    <xsl:choose>
                        <xsl:when test="/parameters/axis='inverted'">
                            <option name="inverted">true</option>
                        </xsl:when>
                        <xsl:when test="/parameters/axis='polar'">
                            <option name="polar">true</option>
                        </xsl:when>
                    </xsl:choose>
                </xsl:if>
            </section>
        </xsl:if>
    </xsl:template>
    <xsl:template match="*" mode="OPTIONSXAXIS" priority="-1">
        <section id="xAxis">
            <option name="categories">
                <xsl:choose>
                    <xsl:when test="/parameters/xAxis/entry">
                        <xsl:attribute name="mode">column</xsl:attribute>
                        <xsl:copy-of select="/parameters/xAxis/entry" />
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:attribute name="mode">unit</xsl:attribute>
                    </xsl:otherwise>
                </xsl:choose>
            </option>
        </section>
    </xsl:template>
    <xsl:template match="*" mode="OPTIONSYAXIS" priority="-1">
        <section id="yAxis" mode="unit" />
    </xsl:template>

    <xsl:template match="*[not(text())]" mode="OPTIONS" priority="-1">
            <xsl:choose>
                <xsl:when test="local-name()='series' and @name">
                    <xsl:message terminate="no">The series name is <xsl:value-of select="@name" /></xsl:message>
                    <section id="{local-name()}" name="{@name}">
                        <xsl:apply-templates mode="OPTIONS"/>
                    </section>
                </xsl:when>
                <xsl:otherwise>
                    <section id="{local-name()}">
                        <xsl:apply-templates mode="OPTIONS"/>
                    </section>
                </xsl:otherwise>
            </xsl:choose>
    </xsl:template>
    <xsl:template match="*[text()]" mode="OPTIONS" priority="-1">
        <option name="{local-name()}">
            <xsl:value-of select="text()"/>
        </option>
    </xsl:template>

    <xsl:template match="yAxis|yAxis2|yAxis3" mode="POSITION">
        <xsl:choose>
            <xsl:when test="self::yAxis">1</xsl:when>
            <xsl:otherwise><xsl:value-of select="substring-after(local-name(),'yAxis')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="yAxis|yAxis2|yAxis3" mode="SERIESYPOSITION">
        <xsl:choose>
            <xsl:when test="self::yAxis">0</xsl:when>
            <xsl:otherwise><xsl:value-of select="substring-after(local-name(),'yAxis')-1"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>